@@ -551,6 +551,36 @@ def test_configurable_keywords():
     ]
 
 
+def test_regexp():
+    s = "column REGEXP '.+static.+'"
+    stmts = sqlparse.parse(s)
+
+    assert len(stmts) == 1
+    assert stmts[0].tokens[0].ttype == T.Keyword
+    assert stmts[0].tokens[0].value == "column"
+    
+    assert stmts[0].tokens[2].ttype == T.Comparison
+    assert stmts[0].tokens[2].value == "REGEXP"
+
+    assert stmts[0].tokens[4].ttype == T.Literal.String.Single
+    assert stmts[0].tokens[4].value == "'.+static.+'"
+
+
+def test_regexp_binary():
+    s = "column REGEXP BINARY '.+static.+'"
+    stmts = sqlparse.parse(s)
+
+    assert len(stmts) == 1
+    assert stmts[0].tokens[0].ttype == T.Keyword
+    assert stmts[0].tokens[0].value == "column"
+    
+    assert stmts[0].tokens[2].ttype == T.Comparison
+    assert stmts[0].tokens[2].value == "REGEXP BINARY"
+
+    assert stmts[0].tokens[4].ttype == T.Literal.String.Single
+    assert stmts[0].tokens[4].value == "'.+static.+'"
+
+
 def test_configurable_regex():
     lex = Lexer.get_default_instance()
     lex.clear()
