@@ -159,9 +159,9 @@ def test_parse_sql_with_binary():
     # See https://github.com/andialbrecht/sqlparse/pull/88
     # digest = '|ËêplL4¡høN{'
     digest = '\x82|\xcb\x0e\xea\x8aplL4\xa1h\x91\xf8N{'
-    sql = "select * from foo where bar = '{}'".format(digest)
+    sql = f"select * from foo where bar = '{digest}'"
     formatted = sqlparse.format(sql, reindent=True)
-    tformatted = "select *\nfrom foo\nwhere bar = '{}'".format(digest)
+    tformatted = f"select *\nfrom foo\nwhere bar = '{digest}'"
     assert formatted == tformatted
 
 
@@ -336,9 +336,9 @@ def test_issue315_utf8_by_default():
         '\x9b\xb2.'
         '\xec\x82\xac\xeb\x9e\x91\xed\x95\xb4\xec\x9a\x94'
     )
-    sql = "select * from foo where bar = '{}'".format(digest)
+    sql = f"select * from foo where bar = '{digest}'"
     formatted = sqlparse.format(sql, reindent=True)
-    tformatted = "select *\nfrom foo\nwhere bar = '{}'".format(digest)
+    tformatted = f"select *\nfrom foo\nwhere bar = '{digest}'"
     assert formatted == tformatted
 
 
